@@ -12,40 +12,41 @@ import tempfile
 import zipfile
 from subprocess import PIPE, STDOUT
 
-from runner import RunnerCore, path_from_root, env_modify, chdir
+from runner import RunnerCore, path_from_root, env_modify
 from runner import create_test_file, ensure_dir, make_executable
-from tools.shared import NODE_JS, PYTHON, EMCC, SPIDERMONKEY_ENGINE, V8_ENGINE
-from tools.shared import CONFIG_FILE, EM_CONFIG, LLVM_ROOT, CANONICAL_TEMP_DIR
-from tools.shared import try_delete
+from tools.config import config_file, EM_CONFIG
+from tools.shared import PYTHON, EMCC
+from tools.shared import CANONICAL_TEMP_DIR
+from tools.shared import try_delete, config
 from tools.shared import EXPECTED_LLVM_VERSION, Cache
-from tools import shared, system_libs
+from tools import shared, system_libs, utils
 
-SANITY_FILE = shared.Cache.get_path('sanity.txt', root=True)
+SANITY_FILE = shared.Cache.get_path('sanity.txt')
 commands = [[EMCC], [PYTHON, path_from_root('tests', 'runner.py'), 'blahblah']]
 
 
 def restore():
-  shutil.copyfile(CONFIG_FILE + '_backup', CONFIG_FILE)
+  shutil.copyfile(config_file + '_backup', config_file)
 
 
 # restore the config file and set it up for our uses
 def restore_and_set_up():
   restore()
-  with open(CONFIG_FILE, 'a') as f:
+  with open(config_file, 'a') as f:
     # make LLVM_ROOT sensitive to the LLVM env var, as we test that
-    f.write('LLVM_ROOT = "%s"\n' % LLVM_ROOT)
+    f.write('LLVM_ROOT = "%s"\n' % config.LLVM_ROOT)
     # unfreeze the cache, so we can test that
     f.write('FROZEN_CACHE = False\n')
 
 
 # wipe the config and sanity files, creating a blank slate
 def wipe():
-  try_delete(CONFIG_FILE)
+  try_delete(config_file)
   try_delete(SANITY_FILE)
 
 
 def add_to_config(content):
-  with open(CONFIG_FILE, 'a') as f:
+  with open(config_file, 'a') as f:
     f.write('\n' + content + '\n')
 
 
@@ -54,16 +55,19 @@ def get_basic_config():
 LLVM_ROOT = "%s"
 BINARYEN_ROOT = "%s"
 NODE_JS = %s
-''' % (LLVM_ROOT, shared.BINARYEN_ROOT, NODE_JS)
+''' % (config.LLVM_ROOT, config.BINARYEN_ROOT, config.NODE_JS)
 
 
-def make_fake_wasm_opt(filename, version):
-  print('make_fake_wasm_opt: %s' % filename)
+def make_fake_tool(filename, version, report_name=None):
+  if not report_name:
+    report_name = os.path.basename(filename)
+  print('make_fake_tool: %s' % filename)
   ensure_dir(os.path.dirname(filename))
   with open(filename, 'w') as f:
     f.write('#!/bin/sh\n')
-    f.write('echo "wasm-opt version %s"\n' % version)
+    f.write('echo "%s version %s"\n' % (report_name, version))
     f.write('echo "..."\n')
+    f.write('exit 0\n')
   make_executable(filename)
 
 
@@ -71,15 +75,8 @@ def make_fake_clang(filename, version):
   """Create a fake clang that only handles --version
   --version writes to stdout (unlike -v which writes to stderr)
   """
-  print('make_fake_clang: %s' % filename)
-  ensure_dir(os.path.dirname(filename))
-  with open(filename, 'w') as f:
-    f.write('#!/bin/sh\n')
-    f.write('echo "clang version %s"\n' % version)
-    f.write('echo "..."\n')
-  shutil.copyfile(filename, filename + '++')
-  make_executable(filename)
-  make_executable(filename + '++')
+  make_fake_tool(filename, version)
+  make_fake_tool(filename + '++', version)
 
 
 def make_fake_llc(filename, targets):
@@ -94,14 +91,6 @@ def make_fake_llc(filename, targets):
   make_executable(filename)
 
 
-def make_fake_lld(filename):
-  print('make_fake_lld: %s' % filename)
-  with open(filename, 'w') as f:
-    f.write('#!/bin/sh\n')
-    f.write('exit 0\n')
-  make_executable(filename)
-
-
 SANITY_MESSAGE = 'Emscripten: Running sanity checks'
 
 EMBUILDER = path_from_root('embuilder.py')
@@ -122,15 +111,15 @@ class sanity(RunnerCore):
     # the sanity checks here
     del os.environ['EMCC_SKIP_SANITY_CHECK']
 
-    assert os.path.exists(CONFIG_FILE), 'To run these tests, we need a (working!) %s file to already exist' % EM_CONFIG
-    shutil.copyfile(CONFIG_FILE, CONFIG_FILE + '_backup')
+    assert os.path.exists(config_file), 'To run these tests, we need a (working!) %s file to already exist' % EM_CONFIG
+    shutil.copyfile(config_file, config_file + '_backup')
 
     print()
     print('Running sanity checks.')
-    print('WARNING: This will modify %s, and in theory can break it although it should be restored properly. A backup will be saved in %s_backup' % (CONFIG_FILE, CONFIG_FILE))
+    print('WARNING: This will modify %s, and in theory can break it although it should be restored properly. A backup will be saved in %s_backup' % (config_file, config_file))
     print()
     print('>>> the original settings file is:')
-    print(open(CONFIG_FILE).read().strip())
+    print(open(config_file).read().strip())
     print('<<<')
     print()
 
@@ -198,7 +187,7 @@ class sanity(RunnerCore):
       finally:
         shutil.rmtree(temp_bin)
 
-      default_config = shared.embedded_config
+      default_config = config.embedded_config
       self.assertContained('Welcome to Emscripten!', output)
       self.assertContained('This is the first time any of the Emscripten tools has been run.', output)
       self.assertContained('A settings file has been copied to %s, at absolute path: %s' % (default_config, default_config), output)
@@ -255,28 +244,30 @@ class sanity(RunnerCore):
 
     # Fake a different llvm version
     restore_and_set_up()
-    with open(CONFIG_FILE, 'a') as f:
+    with open(config_file, 'a') as f:
       f.write('LLVM_ROOT = "' + self.in_dir('fake') + '"')
 
     real_version_x, real_version_y = (int(x) for x in EXPECTED_LLVM_VERSION.split('.'))
     make_fake_llc(self.in_dir('fake', 'llc'), 'wasm32 - WebAssembly 32-bit')
-    make_fake_lld(self.in_dir('fake', 'wasm-ld'))
+    make_fake_tool(self.in_dir('fake', 'wasm-ld'), EXPECTED_LLVM_VERSION)
 
-    with env_modify({'EM_IGNORE_SANITY': '1'}):
-      for inc_x in range(-2, 3):
-        for inc_y in range(-2, 3):
-          expected_x = real_version_x + inc_x
-          expected_y = real_version_y + inc_y
-          if expected_x < 0 or expected_y < 0:
-            continue # must be a valid llvm version
-          print("mod LLVM version: %d %d -> %d %d" % (real_version_x, real_version_x, expected_x, expected_y))
-          make_fake_clang(self.in_dir('fake', 'clang'), '%s.%s' % (expected_x, expected_y))
-          did_modify = inc_x != 0 or inc_y != 0
-          if did_modify:
-            output = self.check_working(EMCC, LLVM_WARNING)
-          else:
-            output = self.check_working(EMCC)
-            self.assertNotContained(LLVM_WARNING, output)
+    for inc_x in range(-2, 3):
+      for inc_y in range(-2, 3):
+        try_delete(SANITY_FILE)
+        expected_x = real_version_x + inc_x
+        expected_y = real_version_y + inc_y
+        if expected_x < 0 or expected_y < 0:
+          continue # must be a valid llvm version
+        print("mod LLVM version: %d %d -> %d %d" % (real_version_x, real_version_y, expected_x, expected_y))
+        make_fake_clang(self.in_dir('fake', 'clang'), '%s.%s' % (expected_x, expected_y))
+        make_fake_tool(self.in_dir('fake', 'llvm-ar'), '%s.%s' % (expected_x, expected_y))
+        make_fake_tool(self.in_dir('fake', 'llvm-nm'), '%s.%s' % (expected_x, expected_y))
+        did_modify = inc_x != 0 or inc_y != 0
+        if did_modify:
+          output = self.check_working(EMCC, LLVM_WARNING)
+        else:
+          output = self.check_working(EMCC)
+          self.assertNotContained(LLVM_WARNING, output)
 
   def test_emscripten_root(self):
     # The correct path
@@ -298,40 +289,40 @@ class sanity(RunnerCore):
     # Clang should report the version number we expect, and emcc should not warn
     assert shared.check_node_version()
     output = self.check_working(EMCC)
-    self.assertNotIn(NODE_WARNING, output)
+    self.assertNotContained(NODE_WARNING, output)
 
     # Fake a different node version
     restore_and_set_up()
-    with open(CONFIG_FILE, 'a') as f:
+    with open(config_file, 'a') as f:
       f.write('NODE_JS = "' + self.in_dir('fake', 'nodejs') + '"')
 
     ensure_dir('fake')
 
-    with env_modify({'EM_IGNORE_SANITY': '1'}):
-      for version, succeed in [('v0.8.0', False),
-                               ('v4.1.0', False),
-                               ('v4.1.1', True),
-                               ('v4.2.3-pre', True),
-                               ('cheez', False)]:
-        print(version, succeed)
-        f = open(self.in_dir('fake', 'nodejs'), 'w')
-        f.write('#!/bin/sh\n')
-        f.write('''if [ $1 = "--version" ]; then
+    for version, succeed in [('v0.8.0', False),
+                             ('v4.1.0', False),
+                             ('v4.1.1', True),
+                             ('v4.2.3-pre', True),
+                             ('cheez', False)]:
+      print(version, succeed)
+      try_delete(SANITY_FILE)
+      f = open(self.in_dir('fake', 'nodejs'), 'w')
+      f.write('#!/bin/sh\n')
+      f.write('''if [ $1 = "--version" ]; then
 echo "%s"
 else
 %s $@
 fi
-''' % (version, NODE_JS))
-        f.close()
-        make_executable(self.in_dir('fake', 'nodejs'))
-        if not succeed:
-          if version[0] == 'v':
-            self.check_working(EMCC, NODE_WARNING)
-          else:
-            self.check_working(EMCC, NODE_WARNING_2)
+''' % (version, ' '.join(config.NODE_JS)))
+      f.close()
+      make_executable(self.in_dir('fake', 'nodejs'))
+      if not succeed:
+        if version[0] == 'v':
+          self.check_working(EMCC, NODE_WARNING)
         else:
-          output = self.check_working(EMCC)
-          assert NODE_WARNING not in output, output
+          self.check_working(EMCC, NODE_WARNING_2)
+      else:
+        output = self.check_working(EMCC)
+        self.assertNotContained(NODE_WARNING, output)
 
   def test_emcc(self):
     SANITY_FAIL_MESSAGE = 'sanity check failed to run'
@@ -370,17 +361,13 @@ fi
     output = self.check_working(EMCC)
     self.assertNotContained(SANITY_MESSAGE, output)
 
-    # also with -v, with or without inputs
-    output = self.check_working([EMCC, '-v'], SANITY_MESSAGE)
-    output = self.check_working([EMCC, '-v'] + MINIMAL_HELLO_WORLD + [], SANITY_MESSAGE)
-
     # Make sure the test runner didn't do anything to the setup
     output = self.check_working(EMCC)
     self.assertNotContained(SANITY_MESSAGE, output)
     self.assertNotContained(SANITY_FAIL_MESSAGE, output)
 
     # emcc should also check sanity if the file is outdated
-    open(CONFIG_FILE, 'a').write('# extra stuff\n')
+    open(config_file, 'a').write('# extra stuff\n')
     output = self.check_working(EMCC)
     self.assertContained(SANITY_MESSAGE, output)
     self.assertNotContained(SANITY_FAIL_MESSAGE, output)
@@ -402,43 +389,42 @@ fi
 
     self.assertContained('hello from emcc with no config file', self.run_js('a.out.js'))
 
-  def erase_cache(self):
-    Cache.erase()
+  def clear_cache(self):
+    self.run_process([EMCC, '--clear-cache'])
     self.assertCacheEmpty()
 
   def assertCacheEmpty(self):
     if os.path.exists(Cache.dirname):
-      self.assertEqual(os.listdir(Cache.dirname), [])
+      # The cache is considered empty if it contains no files at all or just the cache.lock
+      self.assertIn(os.listdir(Cache.dirname), ([], ['cache.lock']))
 
   def ensure_cache(self):
     self.do([EMCC, '-O2', path_from_root('tests', 'hello_world.c')])
 
   def test_emcc_caching(self):
-    BUILDING_MESSAGE = 'generating system library: X'
+    BUILDING_MESSAGE = 'generating system library: %s'
 
     restore_and_set_up()
-    self.erase_cache()
+    self.clear_cache()
 
     # Building a file that *does* need something *should* trigger cache
     # generation, but only the first time
-    libname = 'libc++'
+    libname = Cache.get_lib_name('libc++.a')
     for i in range(3):
       print(i)
       self.clear()
       output = self.do([EMCC, '-O' + str(i), path_from_root('tests', 'hello_libcxx.cpp'), '-s', 'DISABLE_EXCEPTION_CATCHING=0'])
       print('\n\n\n', output)
-      self.assertContainedIf(BUILDING_MESSAGE.replace('X', libname), output, i == 0)
+      self.assertContainedIf(BUILDING_MESSAGE % libname, output, i == 0)
       self.assertContained('hello, world!', self.run_js('a.out.js'))
       self.assertExists(Cache.dirname)
-      full_libname = libname + '.bc' if libname != 'libc++' else libname + '.a'
-      self.assertExists(os.path.join(Cache.dirname, full_libname))
+      self.assertExists(os.path.join(Cache.dirname, libname))
 
   def test_cache_clearing_manual(self):
     # Manual cache clearing
     restore_and_set_up()
     self.ensure_cache()
-    self.assertTrue(os.path.exists(Cache.dirname))
-    self.assertTrue(os.path.exists(Cache.root_dirname))
+    self.assertExists(Cache.dirname)
     output = self.do([EMCC, '--clear-cache'])
     self.assertIn('clearing cache', output)
     self.assertIn(SANITY_MESSAGE, output)
@@ -451,7 +437,7 @@ fi
     make_fake_clang(self.in_dir('fake', 'bin', 'clang'), EXPECTED_LLVM_VERSION)
     make_fake_llc(self.in_dir('fake', 'bin', 'llc'), 'got wasm32 backend! WebAssembly 32-bit')
     with env_modify({'EM_LLVM_ROOT': self.in_dir('fake', 'bin')}):
-      self.assertTrue(os.path.exists(Cache.dirname))
+      self.assertExists(Cache.dirname)
       output = self.do([EMCC])
       self.assertIn('clearing cache', output)
       self.assertCacheEmpty()
@@ -459,18 +445,16 @@ fi
   # FROZEN_CACHE prevents cache clears, and prevents building
   def test_FROZEN_CACHE(self):
     restore_and_set_up()
-    self.erase_cache()
+    self.clear_cache()
     self.ensure_cache()
-    self.assertTrue(os.path.exists(Cache.dirname))
-    self.assertTrue(os.path.exists(Cache.root_dirname))
+    self.assertExists(Cache.dirname)
     # changing config file should not clear cache
     add_to_config('FROZEN_CACHE = True')
     self.do([EMCC])
-    self.assertTrue(os.path.exists(Cache.dirname))
-    self.assertTrue(os.path.exists(Cache.root_dirname))
+    self.assertExists(Cache.dirname)
     # building libraries is disallowed
     output = self.do([EMBUILDER, 'build', 'libemmalloc'])
-    self.assertIn('FROZEN_CACHE disallows building system libs', output)
+    self.assertContained('FROZEN_CACHE is set, but cache file is missing', output)
 
   # Test that if multiple processes attempt to access or build stuff to the
   # cache on demand, that exactly one of the processes will, and the other
@@ -486,6 +470,7 @@ fi
       }
       ''')
     cache_dir_name = self.in_dir('test_cache')
+    libname = Cache.get_lib_name('libc.a')
     with env_modify({'EM_CACHE': cache_dir_name}):
       tasks = []
       num_times_libc_was_built = 0
@@ -494,13 +479,13 @@ fi
         tasks += [p]
       for p in tasks:
         print('stdout:\n', p.stdout)
-        if 'generating system library: libc' in p.stdout:
+        if 'generating system library: ' + libname in p.stdout:
           num_times_libc_was_built += 1
 
     # The cache directory must exist after the build
-    self.assertTrue(os.path.exists(cache_dir_name))
+    self.assertExists(cache_dir_name)
     # The cache directory must contain a built libc
-    self.assertTrue(os.path.exists(os.path.join(cache_dir_name, 'wasm', 'libc.a')))
+    self.assertExists(os.path.join(cache_dir_name, libname))
     # Exactly one child process should have triggered libc build!
     self.assertEqual(num_times_libc_was_built, 1)
 
@@ -509,18 +494,18 @@ fi
 
     fd, custom_config_filename = tempfile.mkstemp(prefix='.emscripten_config_')
 
-    orig_config = open(CONFIG_FILE, 'r').read()
+    orig_config = open(config_file, 'r').read()
 
     # Move the ~/.emscripten to a custom location.
     with os.fdopen(fd, "w") as f:
       f.write(get_basic_config())
 
     # Make a syntax error in the original config file so that attempting to access it would fail.
-    open(CONFIG_FILE, 'w').write('asdfasdfasdfasdf\n\'\'\'' + orig_config)
+    open(config_file, 'w').write('asdfasdfasdfasdf\n\'\'\'' + orig_config)
 
     temp_dir = tempfile.mkdtemp(prefix='emscripten_temp_')
 
-    with chdir(temp_dir):
+    with utils.chdir(temp_dir):
       self.run_process([EMCC, '--em-config', custom_config_filename] + MINIMAL_HELLO_WORLD + ['-O2'])
       result = self.run_js('a.out.js')
 
@@ -534,12 +519,11 @@ fi
     restore_and_set_up()
 
     # listing ports
-
     out = self.do([EMCC, '--show-ports'])
-    assert 'Available ports:' in out, out
-    assert 'SDL2' in out, out
-    assert 'SDL2_image' in out, out
-    assert 'SDL2_net' in out, out
+    self.assertContained('Available ports:', out)
+    self.assertContained('SDL2', out)
+    self.assertContained('SDL2_image', out)
+    self.assertContained('SDL2_net', out)
 
     # using ports
     RETRIEVING_MESSAGE = 'retrieving port'
@@ -554,27 +538,27 @@ fi
         try_delete(PORTS_DIR)
       else:
         self.do([EMCC, '--clear-ports'])
-      assert not os.path.exists(PORTS_DIR)
+      self.assertNotExists(PORTS_DIR)
 
       # Building a file that doesn't need ports should not trigger anything
       output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp')])
       assert RETRIEVING_MESSAGE not in output, output
       assert BUILDING_MESSAGE not in output
       print('no', output)
-      assert not os.path.exists(PORTS_DIR)
+      self.assertNotExists(PORTS_DIR)
 
       def first_use():
         output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
-        assert RETRIEVING_MESSAGE in output, output
-        assert BUILDING_MESSAGE in output, output
+        self.assertContained(RETRIEVING_MESSAGE, output)
+        self.assertContained(BUILDING_MESSAGE, output)
         self.assertExists(PORTS_DIR)
         print('yes', output)
 
       def second_use():
         # Using it again avoids retrieve and build
         output = self.do([EMCC, path_from_root('tests', 'hello_world_sdl.cpp'), '-s', 'USE_SDL=2'])
-        assert RETRIEVING_MESSAGE not in output, output
-        assert BUILDING_MESSAGE not in output, output
+        self.assertNotContained(RETRIEVING_MESSAGE, output)
+        self.assertNotContained(BUILDING_MESSAGE, output)
 
       # Building a file that need a port does trigger stuff
       first_use()
@@ -604,30 +588,30 @@ fi
     test_path = self.in_dir('fake', 'abcd8765')
     ensure_dir(test_path)
 
-    with env_modify({'EM_IGNORE_SANITY': '1'}):
-      jsengines = [('d8',     V8_ENGINE),
-                   ('d8_g',   V8_ENGINE),
-                   ('js',     SPIDERMONKEY_ENGINE),
-                   ('node',   NODE_JS),
-                   ('nodejs', NODE_JS)]
-      for filename, engine in jsengines:
-        if type(engine) is list:
-          engine = engine[0]
-        if not engine:
-          print('WARNING: Not testing engine %s, not configured.' % (filename))
-          continue
+    jsengines = [('d8',     config.V8_ENGINE),
+                 ('d8_g',   config.V8_ENGINE),
+                 ('js',     config.SPIDERMONKEY_ENGINE),
+                 ('node',   config.NODE_JS),
+                 ('nodejs', config.NODE_JS)]
+    for filename, engine in jsengines:
+      try_delete(SANITY_FILE)
+      if type(engine) is list:
+        engine = engine[0]
+      if not engine:
+        print('WARNING: Not testing engine %s, not configured.' % (filename))
+        continue
 
-        print(filename, engine)
+      print(filename, engine)
 
-        test_engine_path = os.path.join(test_path, filename)
-        with open(test_engine_path, 'w') as f:
-          f.write('#!/bin/sh\n')
-          f.write('exec %s $@\n' % (engine))
-        make_executable(test_engine_path)
+      test_engine_path = os.path.join(test_path, filename)
+      with open(test_engine_path, 'w') as f:
+        f.write('#!/bin/sh\n')
+        f.write('exec %s $@\n' % (engine))
+      make_executable(test_engine_path)
 
-        out = self.run_js(sample_script, engine=test_engine_path, args=['--foo'])
+      out = self.run_js(sample_script, engine=test_engine_path, args=['--foo'])
 
-        self.assertEqual('0: --foo', out.strip())
+      self.assertEqual('0: --foo', out.strip())
 
   def test_wacky_env(self):
     restore_and_set_up()
@@ -640,22 +624,22 @@ fi
 
     print('normal build')
     with env_modify({'EMCC_FORCE_STDLIBS': None}):
-      Cache.erase()
+      self.clear_cache()
       build()
       test()
 
     print('wacky env vars, these should not mess our bootstrapping')
     with env_modify({'EMCC_FORCE_STDLIBS': '1'}):
-      Cache.erase()
+      self.clear_cache()
       build()
       test()
 
   def test_vanilla(self):
     restore_and_set_up()
-    Cache.erase()
+    self.clear_cache()
 
     def make_fake(report):
-      with open(CONFIG_FILE, 'a') as f:
+      with open(config_file, 'a') as f:
         f.write('LLVM_ROOT = "' + self.in_dir('fake', 'bin') + '"\n')
         # BINARYEN_ROOT needs to exist in the config, even though this test
         # doesn't actually use it.
@@ -663,7 +647,7 @@ fi
 
       make_fake_clang(self.in_dir('fake', 'bin', 'clang'), EXPECTED_LLVM_VERSION)
       make_fake_llc(self.in_dir('fake', 'bin', 'llc'), report)
-      make_fake_lld(self.in_dir('fake', 'bin', 'wasm-ld'))
+      make_fake_tool(self.in_dir('fake', 'bin', 'wasm-ld'), EXPECTED_LLVM_VERSION)
 
     # fake llc output
 
@@ -679,11 +663,11 @@ fi
     # with no binaryen root, an error is shown
     restore_and_set_up()
 
-    open(CONFIG_FILE, 'a').write('\nBINARYEN_ROOT = ""\n')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is set to empty value in %s' % CONFIG_FILE)
+    open(config_file, 'a').write('\nBINARYEN_ROOT = ""\n')
+    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is set to empty value in %s' % config_file)
 
-    open(CONFIG_FILE, 'a').write('\ndel BINARYEN_ROOT\n')
-    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is not defined in %s' % CONFIG_FILE)
+    open(config_file, 'a').write('\ndel BINARYEN_ROOT\n')
+    self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'BINARYEN_ROOT is not defined in %s' % config_file)
 
   def test_embuilder_force(self):
     restore_and_set_up()
@@ -693,23 +677,31 @@ fi
     # Unless --force is specified
     self.assertContained('generating system library', self.do([EMBUILDER, 'build', 'libemmalloc', '--force']))
 
+  def test_embuilder_force_port(self):
+    restore_and_set_up()
+    self.do([EMBUILDER, 'build', 'zlib'])
+    # Second time it should not generate anything
+    self.assertNotContained('generating port', self.do([EMBUILDER, 'build', 'zlib']))
+    # Unless --force is specified
+    self.assertContained('generating port', self.do([EMBUILDER, 'build', 'zlib', '--force']))
+
   def test_embuilder_wasm_backend(self):
     restore_and_set_up()
     # the --lto flag makes us build wasm-bc
-    self.do([EMCC, '--clear-cache'])
+    self.clear_cache()
     self.run_process([EMBUILDER, 'build', 'libemmalloc'])
-    self.assertExists(os.path.join(shared.CACHE, 'wasm'))
-    self.do([EMCC, '--clear-cache'])
+    self.assertExists(os.path.join(config.CACHE, 'sysroot', 'lib', 'wasm32-emscripten'))
+    self.clear_cache()
     self.run_process([EMBUILDER, 'build', 'libemmalloc', '--lto'])
-    self.assertExists(os.path.join(shared.CACHE, 'wasm-lto'))
+    self.assertExists(os.path.join(config.CACHE, 'sysroot', 'lib', 'wasm32-emscripten', 'lto'))
 
   def test_binaryen_version(self):
     restore_and_set_up()
-    with open(CONFIG_FILE, 'a') as f:
+    with open(config_file, 'a') as f:
       f.write('\nBINARYEN_ROOT = "' + self.in_dir('fake') + '"')
 
-    make_fake_wasm_opt(self.in_dir('fake', 'bin', 'wasm-opt'), 'foo')
+    make_fake_tool(self.in_dir('fake', 'bin', 'wasm-opt'), 'foo')
     self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'error parsing binaryen version (wasm-opt version foo). Please check your binaryen installation')
 
-    make_fake_wasm_opt(self.in_dir('fake', 'bin', 'wasm-opt'), '70')
+    make_fake_tool(self.in_dir('fake', 'bin', 'wasm-opt'), '70')
     self.check_working([EMCC, path_from_root('tests', 'hello_world.c')], 'unexpected binaryen version: 70 (expected ')
